#ifdef WIN32
#define _CRT_SECURE_NO_DEPRECATE
#endif

#include <math.h>
#include "TemplateDatabase.h"

TemplateDatabase::TemplateDatabase()
{
	invFileArray = 0;
	numInvFile = 0;
	N = 0;
	N_ent = 0;
	queryTree = 0;
	forwardFileList = 0;
	scoreTree = 0;
	qNorm = 0.0;
	dNorm = 0.0;
	init = false;
}

TemplateDatabase::~TemplateDatabase()
{
	if(invFileArray != 0)
	{
		delete[] invFileArray;
		invFileArray = 0;
	}
}

void TemplateDatabase::setDatabase( unsigned long int numInvFile, unsigned int numImages)
{
	this->numInvFile = numInvFile;
	N = numImages;
	N_ent = numImages;
}

unsigned int TemplateDatabase::getmi(unsigned int imgID, unsigned long int invFileIndex)
{
	unsigned int mi = 0;

	if(init == false)
		return 0;//database not initialized.

	if(invFileIndex >= numInvFile)
		return 0;//bad inverted file index. Return 0

	if(forwardFileList == 0)
		return 0;//Inverted file not allocated. Return 0

	if(imgID == 0)
		return 0;//imgID 0 is reserved. Return 0

	if(N >= imgID){

		ImgNode tempImgNode = forwardFileList;

		while(tempImgNode != 0){

			if(tempImgNode->imgID == imgID){

				InvFileIndexLink tempInvFileIndexLink = tempImgNode->invFileIndexTree;

				while(tempInvFileIndexLink != 0){

					if(tempInvFileIndexLink->invFileIndex == invFileIndex){
						return tempInvFileIndexLink->mi;
					}

					if( invFileIndex < tempInvFileIndexLink->invFileIndex)
						tempInvFileIndexLink = tempInvFileIndexLink->leftInvFileIndexLink;
					else
						tempInvFileIndexLink = tempInvFileIndexLink->rightInvFileIndexLink;
				}

				return 0;//this should not be reached
			}

			tempImgNode = tempImgNode->nextImgNode;
		}
	}
	else
		return 0;//There are no template images with the specified image ID.

	return mi;
}

unsigned int TemplateDatabase::getLastImgID(unsigned long int invFileIndex)
{
	if(init == false)
		return 0;//database not initialized.

	if(invFileIndex >= numInvFile)
		return 0;//bad index. Return -1

	if(invFileArray[invFileIndex] == 0)
		return 0;//Inverted file not allocated. Return -2

	return (*(invFileArray[invFileIndex]->lastImageLink))->ID;
}

unsigned int TemplateDatabase::getNumberOfImages()
{
	return N;
}

unsigned int TemplateDatabase::getNi(unsigned long int invFileIndex)
{
	if(init == false)
		return -1;//database not initialized.

	if(invFileIndex >= numInvFile)
		return 0;//bad index. Return -1

	if(invFileArray[invFileIndex] == 0)
		return -2;//Inverted file not allocated. Return -2

	return invFileArray[invFileIndex]->Ni;
}

int TemplateDatabase::allocatePointerArray()
{
	if(numInvFile == 0 || init == true)
		return 0;	//return zero if numInvFile is set to zero or if the
					//database is already allocated.

	try
	{
		invFileArray = (InvertedFile*)(new InvertedFileST*[numInvFile]); //allocate the array of pointers

		for(unsigned long int i = 0; i < numInvFile; i++)
		{
			invFileArray[i] = 0;
		}
	}
	catch(char * str)
	{
		cerr << "Exception raised: " << str << '\n';
		init = false;
		return 0;
	}

	init = true;
	return 1;
}

int TemplateDatabase::allocateInvertedFile(unsigned int invFileIndex)
{
	if(init == false)
		return -1;//database not initialized.

	if(invFileIndex >= numInvFile)
		return 0;//bad index. Return -1

	if(invFileArray[invFileIndex] != 0)
	{
		if ( invFileArray[invFileIndex]->imgListHead == 0 )
		{
			invFileArray[invFileIndex]->imgListHead = new ImageLinkST;
			invFileArray[invFileIndex]->imgListHead->ID = 0;
			invFileArray[invFileIndex]->imgListHead->nextImageLink = 0;
			invFileArray[invFileIndex]->lastImageLink = &(invFileArray[invFileIndex]->imgListHead);
		}
		else
			return -2;//Inverted file already allocated. Return -2
	}

	invFileArray[invFileIndex] = new InvertedFileST;
	invFileArray[invFileIndex]->imgListHead = new ImageLinkST;
	invFileArray[invFileIndex]->imgListHead->ID = 0;
	invFileArray[invFileIndex]->imgListHead->nextImageLink = 0;
	invFileArray[invFileIndex]->lastImageLink = &(invFileArray[invFileIndex]->imgListHead);
	invFileArray[invFileIndex]->Ni = 0;

	return 1;//Inverted file allocated. Return 1.
}

int TemplateDatabase::addImageToInvertedFile(unsigned int imgID, unsigned int invFileIndex)
{
	if(init == false)
		return -1;//database not initialized.

	if(invFileIndex >= numInvFile)
		return 0;//bad index. Return -1

	if(invFileArray[invFileIndex] == 0)
		return -2;//Inverted file not allocated. Return -2

	//if the image id == 0(no image has been added yet),
	//add the new image to the inverted file.

	//if ( invFileArray[invFileIndex]->lastImageLink != 0 )
	//{
		if((*(invFileArray[invFileIndex]->lastImageLink))->ID == imgID){
			return 1;
		}
		else if((*(invFileArray[invFileIndex]->lastImageLink))->ID == 0){

			(*(invFileArray[invFileIndex]->lastImageLink))->ID = imgID;
			(*(invFileArray[invFileIndex]->lastImageLink))->nextImageLink = 0;
			invFileArray[invFileIndex]->Ni++;
		}
		else{//else allocate a new link for the new image and add it to the inverted file

			(*(invFileArray[invFileIndex]->lastImageLink))->nextImageLink = new ImageLinkST;
			(*(invFileArray[invFileIndex]->lastImageLink))->nextImageLink->ID = imgID;
			(*(invFileArray[invFileIndex]->lastImageLink))->nextImageLink->nextImageLink = 0;
			invFileArray[invFileIndex]->lastImageLink = &((ImageLink)((*(invFileArray[invFileIndex]->lastImageLink))->nextImageLink));
			invFileArray[invFileIndex]->Ni++;
		}
	//}

	return 1;
}

int TemplateDatabase::addNi(unsigned int invFileIndex, unsigned int addNi)
{
	if(init == false)
		return -1;//database not initialized.

	if(invFileIndex >= numInvFile)
		return 0;//bad index. Return -1

	if(invFileArray[invFileIndex] == 0)
		return -2;//Inverted file not allocated. Return -2

	invFileArray[invFileIndex]->Ni += addNi;

	return 1;
}

int TemplateDatabase::incramentmi(unsigned int imgID, unsigned long int invFileIndex)
{
	if(init == false)
		return -1;//database not initialized.

	if(invFileIndex >= numInvFile)
		return 0;//bad inverted file index. Return -1

	if(forwardFileList == 0)
		return -2;//forwardFileList not allocated. Return -2


	if(N >= imgID){

		ImgNode tempImgNode = forwardFileList;

		while(tempImgNode != 0){//find the forward file for the image while another link in the list exists

			if(tempImgNode->imgID == imgID){//forward file found if this is true

				InvFileIndexLink tempInvFileIndexLink = tempImgNode->invFileIndexTree;

				while(tempInvFileIndexLink != 0){//find the invertd file index whos mi will be incramented

					if(tempInvFileIndexLink->invFileIndex == invFileIndex){//inverted file index found if this is true
						tempInvFileIndexLink->mi++;
						return 1;
					}

					if(tempInvFileIndexLink->invFileIndex < invFileIndex)
						tempInvFileIndexLink = tempInvFileIndexLink->leftInvFileIndexLink;
					else
						tempInvFileIndexLink = tempInvFileIndexLink->rightInvFileIndexLink;
				}

				return 0;//this should not be reached
			}

			tempImgNode = tempImgNode->nextImgNode;
		}
	}
	else
		return 0;//There are no template images with the specified image ID.

	return 1;
}

void TemplateDatabase::incramentNumberOfImages()
{
	N++;
	N_ent++;
}

void TemplateDatabase::updateQueryTree(unsigned int invFileIndex, unsigned int descIndex)
{
	if(queryTree == 0)
	{
		queryTree = new QueryNodeST;

		queryTree->LeafNodeID = invFileIndex;
		queryTree->ni = 1;

		queryTree->descList = new DescIndexST;
		queryTree->descList->index = descIndex;
		queryTree->descList->nextDescIndex = 0;
		queryTree->descListEnd = queryTree->descList;

		queryTree->leftQueryNode = 0;
		queryTree->rightQueryNode = 0;
	}
	else
	{
		updateQuery(queryTree, invFileIndex, descIndex);
	}

	if(invFileArray[invFileIndex] != 0)
	{
		ImageLink tempImageLink = invFileArray[invFileIndex]->imgListHead;
		unsigned int imgID;

		for(;;)
		{
			if(tempImageLink == 0)
				break;
			else
			{
				imgID = tempImageLink->ID;
				updateScoreTree(imgID, 0);
				tempImageLink = tempImageLink->nextImageLink;
			}
		}
	}

	return;
}

void TemplateDatabase::updateQuery(QueryNode queryNode, unsigned long int invFileIndex, unsigned int descIndex)
{
	QueryNode tempQueryNode = queryNode;

	for(;;)
	{
		if(tempQueryNode == 0)
		{
			tempQueryNode = new QueryNodeST;

			tempQueryNode->LeafNodeID = invFileIndex;
			tempQueryNode->ni = 1;

			tempQueryNode->descList = new DescIndexST;
			tempQueryNode->descList->index = descIndex;
			tempQueryNode->descList->nextDescIndex = 0;
			tempQueryNode->descListEnd = tempQueryNode->descList;

			tempQueryNode->leftQueryNode = 0;
			tempQueryNode->rightQueryNode = 0;
			break;
		}
		else
		{
			if(invFileIndex == tempQueryNode->LeafNodeID)
			{
				tempQueryNode->ni++;

				tempQueryNode->descListEnd->nextDescIndex = new DescIndexST;
				tempQueryNode->descListEnd->nextDescIndex->index = descIndex;
				tempQueryNode->descListEnd->nextDescIndex->nextDescIndex = 0;
				tempQueryNode->descListEnd = tempQueryNode->descListEnd->nextDescIndex;

				break;
			}
			else if(invFileIndex < tempQueryNode->LeafNodeID){

				if(tempQueryNode->leftQueryNode == 0){

					tempQueryNode->leftQueryNode = new QueryNodeST;
					tempQueryNode->leftQueryNode->LeafNodeID = invFileIndex;
					tempQueryNode->leftQueryNode->ni = 1;

					tempQueryNode->leftQueryNode->descList = new DescIndexST;
					tempQueryNode->leftQueryNode->descList->index = descIndex;
					tempQueryNode->leftQueryNode->descList->nextDescIndex = 0;
					tempQueryNode->leftQueryNode->descListEnd = tempQueryNode->leftQueryNode->descList;

					tempQueryNode->leftQueryNode->leftQueryNode = 0;
					tempQueryNode->leftQueryNode->rightQueryNode = 0;
					break;
				}

				tempQueryNode = tempQueryNode->leftQueryNode;
			}
			else{

				if(tempQueryNode->rightQueryNode == 0){

					tempQueryNode->rightQueryNode = new QueryNodeST;
					tempQueryNode->rightQueryNode->LeafNodeID = invFileIndex;
					tempQueryNode->rightQueryNode->ni = 1;

					tempQueryNode->rightQueryNode->descList = new DescIndexST;
					tempQueryNode->rightQueryNode->descList->index = descIndex;
					tempQueryNode->rightQueryNode->descList->nextDescIndex = 0;
					tempQueryNode->rightQueryNode->descListEnd = tempQueryNode->rightQueryNode->descList;

					tempQueryNode->rightQueryNode->leftQueryNode = 0;
					tempQueryNode->rightQueryNode->rightQueryNode = 0;
					break;
				}

				tempQueryNode = tempQueryNode->rightQueryNode;
			}
		}
	}

	return;
}

unsigned int TemplateDatabase::searchQueryTree(unsigned long int invFileIndex)
{
	unsigned int ni = 0;

	QueryNode *tempQueryNode = &queryTree;

	for(;;)
	{
		if((*tempQueryNode) == 0)
			break;
		else if((*tempQueryNode)->LeafNodeID == invFileIndex)
		{
			ni = (*tempQueryNode)->ni;
			break;
		}
		else if(invFileIndex < (*tempQueryNode)->LeafNodeID)
			tempQueryNode = &((QueryNode)((*tempQueryNode)->leftQueryNode));
		else
			tempQueryNode = &((QueryNode)((*tempQueryNode)->rightQueryNode));
	}

	return ni;
}

void TemplateDatabase::deleteQuery()
{
	if(queryTree == 0)
	{
		return;
	}
	else
	{
		deleteQueryTree(queryTree);
		delete queryTree;
		queryTree = 0;
	}

	if(scoreTree == 0)
	{
		return;
	}
	else
	{
		deleteScoreTree(scoreTree);
		delete scoreTree;
		scoreTree = 0;
	}

	qNorm = 0.0;
	dNorm = 0.0;

	return;
}

void TemplateDatabase::deleteQueryTree(QueryNode queryNode)
{
	if(queryNode == 0)
		return;
	else
	{
		deleteQueryTree(queryNode->leftQueryNode);
		if(queryNode->leftQueryNode != 0)
			delete (queryNode->leftQueryNode);

		deleteQueryTree(queryNode->rightQueryNode);
		if(queryNode->rightQueryNode != 0)
			delete (queryNode->rightQueryNode);
	}

	return;
}

void TemplateDatabase::deleteScoreTree(ScoreNode scoreNode)
{
	if(scoreNode == 0)
		return;
	else
	{
		deleteScoreTree(scoreNode->leftScoreNode);
		if(scoreNode->leftScoreNode != 0)
			delete (scoreNode->leftScoreNode);

		deleteScoreTree(scoreNode->rightScoreNode);
		if(scoreNode->rightScoreNode != 0)
			delete (scoreNode->rightScoreNode);
	}

	return;
}

void TemplateDatabase::updateObjectIDS(unsigned int imgID, unsigned int nextImgID){

	ImgNode tempImgNode = forwardFileList;

	while(tempImgNode != 0)
	{
		if(tempImgNode->imgID == imgID){

			if(tempImgNode->objectIDSList == 0){
				tempImgNode->objectIDSList = new ObjectIDSST;
				tempImgNode->objectIDSList->ID = nextImgID;
				tempImgNode->objectIDSList->nextObjectIDS = 0;
				tempImgNode->objectIDSListEnd = tempImgNode->objectIDSList;
				break;
			}

			tempImgNode->objectIDSListEnd->nextObjectIDS = new ObjectIDSST;
			tempImgNode->objectIDSListEnd->nextObjectIDS->ID = imgID;
			tempImgNode->objectIDSListEnd->nextObjectIDS->nextObjectIDS = 0;
			tempImgNode->objectIDSListEnd = tempImgNode->objectIDSListEnd->nextObjectIDS;
			break;

		}

		tempImgNode = tempImgNode->nextImgNode;
	}

}

vector<int> TemplateDatabase::getObjectIDS(unsigned int imgID){

	ImgNode tempImgNode = forwardFileList;
	vector<int> objectList(1,-1);

	while(tempImgNode != 0)
	{
		if(tempImgNode->imgID == imgID){

			ObjectIDS tempObjectIDS = tempImgNode->objectIDSList;
			int i = 1;

			while(tempObjectIDS != 0){

				objectList.resize(i);
				objectList.at(i-1) = tempObjectIDS->ID;

				i++;
				tempObjectIDS = tempObjectIDS->nextObjectIDS;
			}

			break;
		}

		tempImgNode = tempImgNode->nextImgNode;
	}

	return objectList;

}

void TemplateDatabase::updateForwardFileTree(unsigned int imgID, unsigned int invFileIndex)
{
	if(forwardFileList == 0)
	{
		forwardFileList = new ImgNodeST;

		forwardFileList->imgID = imgID;
		forwardFileList->invFileIndexTree = new InvFileIndexLinkST;
		forwardFileList->invFileIndexTree->invFileIndex = invFileIndex;
		forwardFileList->invFileIndexTree->mi = 1;
		forwardFileList->objectIDSList = 0;
		forwardFileList->objectIDSListEnd = 0;
		forwardFileList->invFileIndexTree->leftInvFileIndexLink = 0;
		forwardFileList->invFileIndexTree->rightInvFileIndexLink = 0;
		forwardFileList->nextImgNode = 0;
	}
	else
	{
		ImgNode tempImgNode = forwardFileList;

		for(;;)
		{
			if(tempImgNode == 0)
			{
				tempImgNode = new ImgNodeST;

				tempImgNode->imgID = imgID;
				tempImgNode->invFileIndexTree = new InvFileIndexLinkST;
				tempImgNode->invFileIndexTree->invFileIndex = invFileIndex;
				tempImgNode->invFileIndexTree->mi = 1;
				tempImgNode->invFileIndexTree->leftInvFileIndexLink = 0;
				tempImgNode->invFileIndexTree->rightInvFileIndexLink = 0;
				tempImgNode->nextImgNode = 0;
				tempImgNode->objectIDSList = 0;
				tempImgNode->objectIDSListEnd = 0;

				break;
			}
			else
			{
				if(tempImgNode->imgID == imgID)
				{
					if(tempImgNode->invFileIndexTree->invFileIndex == invFileIndex){
						tempImgNode->invFileIndexTree->mi++;
						break;
					}
					else
					{
						InvFileIndexLink tempInvFileIndexLink = tempImgNode->invFileIndexTree;

						for(;;)
						{
							if(tempInvFileIndexLink == 0)
							{
								tempInvFileIndexLink = new InvFileIndexLinkST;
								tempInvFileIndexLink->invFileIndex = invFileIndex;
								tempInvFileIndexLink->mi = 1;
								tempInvFileIndexLink->leftInvFileIndexLink = 0;
								tempInvFileIndexLink->rightInvFileIndexLink = 0;
								return;
							}
							else if(tempInvFileIndexLink->invFileIndex == invFileIndex){
								tempInvFileIndexLink->mi++;
								return;
							}
							else if(invFileIndex < tempInvFileIndexLink->invFileIndex){

								if(tempInvFileIndexLink->leftInvFileIndexLink == 0){
									tempInvFileIndexLink->leftInvFileIndexLink = new InvFileIndexLinkST;
									tempInvFileIndexLink->leftInvFileIndexLink->invFileIndex = invFileIndex;
									tempInvFileIndexLink->leftInvFileIndexLink->mi = 1;
									tempInvFileIndexLink->leftInvFileIndexLink->leftInvFileIndexLink = 0;
									tempInvFileIndexLink->leftInvFileIndexLink->rightInvFileIndexLink = 0;
									return;
								}

								tempInvFileIndexLink = tempInvFileIndexLink->leftInvFileIndexLink;
							}
							else{

								if(tempInvFileIndexLink->rightInvFileIndexLink == 0){
									tempInvFileIndexLink->rightInvFileIndexLink = new InvFileIndexLinkST;
									tempInvFileIndexLink->rightInvFileIndexLink->invFileIndex = invFileIndex;
									tempInvFileIndexLink->rightInvFileIndexLink->mi = 1;
									tempInvFileIndexLink->rightInvFileIndexLink->leftInvFileIndexLink = 0;
									tempInvFileIndexLink->rightInvFileIndexLink->rightInvFileIndexLink = 0;
									return;
								}

								tempInvFileIndexLink = tempInvFileIndexLink->rightInvFileIndexLink;
							}
						}
					}
				}
				else{

					if( tempImgNode->nextImgNode == 0){

						tempImgNode->nextImgNode = new ImgNodeST;

						tempImgNode->nextImgNode->imgID = imgID;
						tempImgNode->nextImgNode->invFileIndexTree = new InvFileIndexLinkST;
						tempImgNode->nextImgNode->invFileIndexTree->invFileIndex = invFileIndex;
						tempImgNode->nextImgNode->invFileIndexTree->mi = 1;
						tempImgNode->nextImgNode->invFileIndexTree->leftInvFileIndexLink = 0;
						tempImgNode->nextImgNode->invFileIndexTree->rightInvFileIndexLink = 0;
						tempImgNode->nextImgNode->nextImgNode = 0;
						tempImgNode->nextImgNode->objectIDSList = 0;
						tempImgNode->nextImgNode->objectIDSListEnd = 0;

						break;
					}

					tempImgNode = tempImgNode->nextImgNode;
				}
			}
		}
	}

	return;
}

void TemplateDatabase::updateScoreTree(unsigned int imgID, double score)
{
	if(scoreTree == 0)
	{
		scoreTree = new ScoreNodeST;
		scoreTree->imgID = imgID;
		scoreTree->indexSet = 0;
		scoreTree->indexSetEnd = 0;
		scoreTree->leftScoreNode = 0;
		scoreTree->rightScoreNode = 0;
		scoreTree->score = score;
	}
	else
	{
		ScoreNode tempScoreNode = scoreTree;

		for(;;)
		{
			if(tempScoreNode == 0)
			{
				tempScoreNode = new ScoreNodeST;
				tempScoreNode->imgID = imgID;
				tempScoreNode->indexSet = 0;
				tempScoreNode->indexSetEnd = 0;
				tempScoreNode->leftScoreNode = 0;
				tempScoreNode->rightScoreNode = 0;
				tempScoreNode->score = score;
				break;
			}
			else if(tempScoreNode->imgID == imgID)
			{
				tempScoreNode->score += score;
				break;
			}
			else if(imgID < tempScoreNode->imgID){

				if(tempScoreNode->leftScoreNode == 0){
					tempScoreNode->leftScoreNode = new ScoreNodeST;
					tempScoreNode->leftScoreNode->imgID = imgID;
					tempScoreNode->leftScoreNode->indexSet = 0;
					tempScoreNode->leftScoreNode->indexSetEnd = 0;
					tempScoreNode->leftScoreNode->leftScoreNode = 0;
					tempScoreNode->leftScoreNode->rightScoreNode = 0;
					tempScoreNode->leftScoreNode->score = score;
					break;
				}

				tempScoreNode = tempScoreNode->leftScoreNode;
			}
			else{

				if(tempScoreNode->rightScoreNode == 0){
					tempScoreNode->rightScoreNode = new ScoreNodeST;
					tempScoreNode->rightScoreNode->imgID = imgID;
					tempScoreNode->rightScoreNode->indexSet = 0;
					tempScoreNode->rightScoreNode->indexSetEnd = 0;
					tempScoreNode->rightScoreNode->leftScoreNode = 0;
					tempScoreNode->rightScoreNode->rightScoreNode = 0;
					tempScoreNode->rightScoreNode->score = score;
					break;
				}

				tempScoreNode = tempScoreNode->rightScoreNode;
			}
		}
	}

	return;
}

double TemplateDatabase::getScore(unsigned int imgID)
{
	ScoreNode tempScoreNode = scoreTree;

	for(;;)
	{
		if(tempScoreNode == 0)
			break;
		else if(tempScoreNode->imgID == imgID)
			return tempScoreNode->score;
		else if(imgID < tempScoreNode->imgID)
			tempScoreNode = tempScoreNode->leftScoreNode;
		else
			tempScoreNode = tempScoreNode->rightScoreNode;
	}

	return 1.7976931348623158e+308;//If the image was not found, return max score value.
}

void TemplateDatabase::getImgScoreList(vector<int>& idVector)
{
	getImgScoreIDS(scoreTree, idVector);
}

void TemplateDatabase::getImgScoreIDS(ScoreNode tempScoreNode, vector<int>& idVector)
{
	if(tempScoreNode == 0)
		return;

	getImgScoreIDS(tempScoreNode->leftScoreNode, idVector);

	getImgScoreIDS(tempScoreNode->rightScoreNode, idVector);

	int vectorSize = idVector.size();
	idVector.resize(vectorSize+1);
	idVector.at(vectorSize+1) = tempScoreNode->imgID;
}

vector<int> TemplateDatabase::getDescIndexList(unsigned int imgID)
{
	ScoreNode tempScoreNode = scoreTree;
	vector<int> list(1,-1);

	for(;;)
	{
		if(tempScoreNode == 0)
			break;
		else if(tempScoreNode->imgID == imgID){

			DescIndexSet tempDescIndexSet = tempScoreNode->indexSet;
			int i = 0;

			while(tempDescIndexSet != 0){

				DescIndex tempDescIndex = tempDescIndexSet->indexList;

				while(tempDescIndex != 0){

					i++;
					list.resize(i);

					list.at(i-1) = tempDescIndex->index;

					tempDescIndex = tempDescIndex->nextDescIndex;
				}

				tempDescIndexSet = tempDescIndexSet->nextIndexList;
			}

			break;
		}
		else if(imgID < tempScoreNode->imgID)
			tempScoreNode = tempScoreNode->leftScoreNode;
		else
			tempScoreNode = tempScoreNode->rightScoreNode;
	}

	return list;//If the image was not found, the vector will contain -1.
}

void TemplateDatabase::calcImgScore(ScoreNode tempScoreNode)//This function will calculate scores for all the images in the score list.
{
	if(tempScoreNode != 0)
	{
		calcImgScore(tempScoreNode->leftScoreNode);

		calcImgScore(tempScoreNode->rightScoreNode);

		calcScore(tempScoreNode);//start to calculate the score for this image.
		tempScoreNode->score += 2;//finally add 2 to the score of this image.
	}

	return;
}

void TemplateDatabase::calcScore(ScoreNode tempScoreNode)//This function will partially calculate a score for one image.
//Once this function is done, 2 should be added to the score of the image.
{
	ImgNode tempImgNode = forwardFileList;

	//Find the forward file for the scoring image.
	for(unsigned int i = 1; i <= N; i++){

		if(tempImgNode->imgID == tempScoreNode->imgID)
			break;

		tempImgNode = tempImgNode->nextImgNode;
	}

	InvFileIndexLink tempInvFileIndexLink = tempImgNode->invFileIndexTree;//get a temporary pointer to the list of inverted file indexes
	//that the template images descriptor vectors went to.
	//reset dNorm for this image
	dNorm = 0.0;
	calcDNorm(tempInvFileIndexLink);

	calcQueryScore(queryTree, tempInvFileIndexLink, tempScoreNode);//start to calculate the score for this image

	return;
}

void TemplateDatabase::calcQueryScore(QueryNode tempQueryNode, InvFileIndexLink tempInvFileIndexLink, ScoreNode tempScoreNode)
{
	if(tempQueryNode != 0)
	{
		calcQueryScore(tempQueryNode->leftQueryNode, tempInvFileIndexLink, tempScoreNode);

		calcQueryScore(tempQueryNode->rightQueryNode, tempInvFileIndexLink, tempScoreNode);

		queryInvFileScore(tempInvFileIndexLink, tempQueryNode, tempScoreNode);
	}

	return;
}

void TemplateDatabase::queryInvFileScore(InvFileIndexLink tempInvFileIndexLink, QueryNode tempQueryNode, ScoreNode tempScoreNode)
{
	if(tempInvFileIndexLink == 0)
		return;
	else if(tempInvFileIndexLink->invFileIndex == tempQueryNode->LeafNodeID)
	{

		double wi = getNodeWeight(tempQueryNode->LeafNodeID);
		double qi = tempQueryNode->ni*wi;
		double mi = getmi(tempScoreNode->imgID, tempQueryNode->LeafNodeID);
		double di = mi*wi;

		tempScoreNode->score += abs((qi/qNorm) - (di/dNorm));
		tempScoreNode->score -= abs(qi/qNorm);
		tempScoreNode->score -= abs(di/dNorm);

		if(tempScoreNode->indexSet == 0){
			tempScoreNode->indexSet = new DescIndexSetST;
			tempScoreNode->indexSet->indexList = tempQueryNode->descList;
			tempScoreNode->indexSet->nextIndexList = 0;
			tempScoreNode->indexSetEnd = tempScoreNode->indexSet;
		}
		else{
			tempScoreNode->indexSetEnd->nextIndexList = new DescIndexSetST;
			tempScoreNode->indexSetEnd->nextIndexList->indexList = tempQueryNode->descList;
			tempScoreNode->indexSetEnd->nextIndexList->nextIndexList = 0;
			tempScoreNode->indexSetEnd = tempScoreNode->indexSetEnd->nextIndexList;
		}

		return;
	}
	else if(tempQueryNode->LeafNodeID < tempInvFileIndexLink->invFileIndex)
		queryInvFileScore(tempInvFileIndexLink->leftInvFileIndexLink, tempQueryNode, tempScoreNode);
	else
		queryInvFileScore(tempInvFileIndexLink->rightInvFileIndexLink, tempQueryNode, tempScoreNode);

	return;
}

unsigned int TemplateDatabase::getMatch()
{
	unsigned int matchID = 0;

	calcQNorm(queryTree);

	calcImgScore(scoreTree);

	for(unsigned int i = 1; i <= N; i++)
	{
		if(i == 1)
			matchID = 1;
		else
		{
			if(getScore(i) < getScore(matchID))
				matchID = i;
		}
	}

	return matchID;
}

void TemplateDatabase::calcQNorm(QueryNode tempQueryNode)
{
	if( tempQueryNode != 0 )
	{
		calcQNorm(tempQueryNode->leftQueryNode);
		calcQNorm(tempQueryNode->rightQueryNode);

		double wi = getNodeWeight(tempQueryNode->LeafNodeID);

		double ni = tempQueryNode->ni;
		qNorm += abs(ni*wi);
	}
}

void TemplateDatabase::calcDNorm(InvFileIndexLink tempInvFileIndexLink)
{
	if( tempInvFileIndexLink != 0 )
	{
		calcDNorm(tempInvFileIndexLink->leftInvFileIndexLink);
		calcDNorm(tempInvFileIndexLink->rightInvFileIndexLink);

		double wi = getNodeWeight(tempInvFileIndexLink->invFileIndex);

		double mi = tempInvFileIndexLink->mi;
		dNorm += abs(mi*wi);
	}
}

double TemplateDatabase::getNodeWeight(unsigned long int invFileIndex)
{
	double Ni = 1;

	if(invFileArray[invFileIndex] != 0)
		Ni = invFileArray[invFileIndex]->Ni;

	return log(((double)N_ent)/Ni);
}

void TemplateDatabase::saveInvertedFile(ofstream& myfile){

	myfile << numInvFile << " " << N << " " << N_ent << endl;

	for(unsigned int i = 0; i < numInvFile; i++){
		if(invFileArray[i] != 0){

			myfile << i << " " << invFileArray[i]->Ni;

			ImageLink tempImageLink = invFileArray[i]->imgListHead;

			while(tempImageLink != 0){

				if(tempImageLink->ID != 0)
					myfile  << " " << tempImageLink->ID;

				tempImageLink = tempImageLink->nextImageLink;
			}

			myfile << endl;
		}
	}

	myfile << numInvFile << endl;
}

void TemplateDatabase::loadInvertedFile(FILE *myfile){

	fscanf( myfile , "%d %d %d" , &numInvFile, &N, &N_ent );

	unsigned long invertedFile;
	unsigned long tempNi = 0;

	fscanf( myfile , "%d" , &invertedFile);

	while(invertedFile != numInvFile){

		fscanf( myfile , " %d" , &tempNi);

		unsigned long imgID = 0;
		char lastChar;

		fscanf( myfile , "%c" , &lastChar );

		if(lastChar == ' '){

			while(lastChar == ' '){

				fscanf( myfile , "%d" , &imgID );

				allocateInvertedFile(invertedFile);
				addImageToInvertedFile(imgID, invertedFile);
				tempNi--;

				fscanf( myfile , "%c" , &lastChar );
			}

			addNi(invertedFile, tempNi);
		}
		else{
			invFileArray[invertedFile] = new InvertedFileST;
	invFileArray[invertedFile]->imgListHead = NULL; //new ImageLinkST;
//	invFileArray[invFileIndex]->imgListHead->ID = 0;
//	invFileArray[invFileIndex]->imgListHead->nextImageLink = 0;
	invFileArray[invertedFile]->lastImageLink = NULL; //&(invFileArray[invFileIndex]->imgListHead);
	invFileArray[invertedFile]->Ni = 0;

			//allocateInvertedFile(invertedFile);
			addNi(invertedFile, tempNi);
		}

		fscanf( myfile , "%d" , &invertedFile );
	}
}

void TemplateDatabase::loadForwardFile(FILE *myfile){

	unsigned int tempN;

	fscanf( myfile , "%d/n" , &tempN );

	if(tempN != 0){

		unsigned long imgID = 0;

		fscanf( myfile , "%d" , &imgID );

		while(imgID <= N){

			int invertedFile = 0;
			char lastChar;

			fscanf( myfile , "%c" , &lastChar );

			while(lastChar == ' '){

				int mi;
				fscanf( myfile , "%d:%d" , &invertedFile, &mi );

				if(invertedFile != -1 && mi != -1){
					for(int ii = 0; ii < mi; ii++)
						updateForwardFileTree(imgID, (unsigned)invertedFile);
				}

				fscanf( myfile , "%c" , &lastChar );
			}

			if(imgID == N)
				break;

			fscanf( myfile , "%d" , &imgID );
		}
	}
}

void TemplateDatabase::saveForwardFile(ofstream& myfile){

	ImgNode tempImgNode = forwardFileList;

	if(forwardFileList == 0)
		myfile << 0;
	else{
		myfile << N << endl;

		while(tempImgNode != 0){

			myfile << tempImgNode->imgID << " ";

			saveInvFileIndexTree(tempImgNode->invFileIndexTree, myfile);

			myfile << "-1:-1";

			myfile << endl;

			tempImgNode = tempImgNode->nextImgNode;
		}
	}
}

void TemplateDatabase::saveInvFileIndexTree(InvFileIndexLink tempInvFileIndexLink, ofstream& myfile){

	if(tempInvFileIndexLink != 0){

		myfile << tempInvFileIndexLink->invFileIndex << ":" << tempInvFileIndexLink->mi << " ";

		saveInvFileIndexTree(tempInvFileIndexLink->leftInvFileIndexLink, myfile);
		saveInvFileIndexTree(tempInvFileIndexLink->rightInvFileIndexLink, myfile);
	}
}

void TemplateDatabase::saveDatabase(){
	ofstream myfile;
	myfile.open("database.txt");

	saveInvertedFile(myfile);

	saveForwardFile(myfile);

	myfile.close();
}

void TemplateDatabase::loadDatabase(char *fileName){

	FILE *myfile;
	myfile = fopen( fileName , "r" );

	loadInvertedFile(myfile);
	loadForwardFile(myfile);

	fclose(myfile);
}

void TemplateDatabase::resetInvertedFile(unsigned long int invFileIndex)
{
	if (invFileArray[invFileIndex] != 0)
	{
		invFileArray[invFileIndex]->imgListHead->ID = 0;

		deleteImageLinkList(invFileArray[invFileIndex]->imgListHead->nextImageLink);

		if(invFileArray[invFileIndex]->imgListHead->nextImageLink != 0){
			delete invFileArray[invFileIndex]->imgListHead->nextImageLink;
			invFileArray[invFileIndex]->imgListHead->nextImageLink = 0;
		}

		invFileArray[invFileIndex]->lastImageLink = &(invFileArray[invFileIndex]->imgListHead);
	}
}

void TemplateDatabase::resetForwardFileList()
{
	if (forwardFileList != 0){

		deleteImgNode(forwardFileList);

		if(forwardFileList != 0){
			delete forwardFileList;
			forwardFileList = 0;
		}
	}

	return;
}

void TemplateDatabase::deleteInvFileIndexLink(InvFileIndexLink tree)
{
	if(tree == 0)
		return;
	else
	{
		deleteInvFileIndexLink(tree->leftInvFileIndexLink);
		if(tree->leftInvFileIndexLink != 0){
			delete tree->leftInvFileIndexLink;
			tree->leftInvFileIndexLink = 0;
		}

		deleteInvFileIndexLink(tree->rightInvFileIndexLink);
		if(tree->rightInvFileIndexLink != 0){
			delete tree->rightInvFileIndexLink;
			tree->rightInvFileIndexLink = 0;
		}
	}

	return;
}

void TemplateDatabase::deleteImgNode(ImgNode tempImgNode)
{
	if(tempImgNode == 0)
		return;
	else
	{
		deleteObjectIDS(tempImgNode->objectIDSList);

		if(tempImgNode->objectIDSList != 0){
			delete tempImgNode->objectIDSList;
			tempImgNode->objectIDSList = 0;
			tempImgNode->objectIDSListEnd = 0;
		}

		deleteInvFileIndexLink(tempImgNode->invFileIndexTree);

		if(tempImgNode->invFileIndexTree != 0){
			delete tempImgNode->invFileIndexTree;
			tempImgNode->invFileIndexTree = 0;
		}

		deleteImgNode(tempImgNode->nextImgNode);

		if(tempImgNode->nextImgNode != 0){
			delete tempImgNode->nextImgNode;
			tempImgNode->nextImgNode = 0;
		}
	}

	return;
}

void TemplateDatabase::deleteObjectIDS(ObjectIDS tempObjectIDS)
{
	if(tempObjectIDS == 0)
		return;
	else
	{
		deleteObjectIDS(tempObjectIDS->nextObjectIDS);

		if(tempObjectIDS->nextObjectIDS != 0){
			delete tempObjectIDS->nextObjectIDS;
			tempObjectIDS->nextObjectIDS = 0;
		}
	}

	return;
}

void TemplateDatabase::deleteImageLinkList(ImageLink tempImageLink)
{
	if(tempImageLink == 0)
		return;
	else
	{
		deleteImageLinkList(tempImageLink->nextImageLink);

		if(tempImageLink->nextImageLink != 0){
			delete tempImageLink->nextImageLink;
			tempImageLink->nextImageLink = 0;
		}
	}

	return;
}