#ifndef VOCABULARYTREE_H
#define VOCABULARYTREE_H

using namespace std;

class VocabularyTree
{
public:
	VocabularyTree();
	VocabularyTree(unsigned char dim, unsigned char levels, unsigned char clusters);
	~VocabularyTree();
	void setDimension(unsigned char dim);
	void setLevel(unsigned char levels);
	void setNumberOfClusters(unsigned char clusters);
	void initTree(unsigned char dim, unsigned char levels, unsigned char clusters);
	int allocateTree();
	unsigned char getDescVectorElement(unsigned char levelIndex, unsigned int nodeIndex, unsigned char elementIndex);
	unsigned char* getDescVector(unsigned char levelIndex, unsigned int nodeIndex);
	void setDescVectorElement(unsigned char levelIndex, unsigned int nodeIndex, unsigned char elementIndex, unsigned char element);
	void setDescVector(unsigned char levelIndex, unsigned int nodeIndex, unsigned char* descVector);
	unsigned int getParentArrayIndex(unsigned char levelIndex, unsigned int nodeIndex);
	unsigned int getChildArrayIndex(unsigned char levelIndex, unsigned int nodeIndex, unsigned char child);
    //void writeTreeFile( char *file_loc );
    //void retrieveTreeIntoMem( char *file_loc );
    int calcNumNodes();
	void setNodeToNull(unsigned char levelIndex, unsigned int nodeIndex);
	bool nodeExist(unsigned char levelIndex, unsigned int nodeIndex);
	void allocateNode(unsigned char levelIndex, unsigned int nodeIndex);
    long double total_nodes;
	void binaryModeWriteTreeFile( char *file_loc );
    void binaryModeRetrieveTreeIntoMem( char *file_loc );
	void textModeRetrieveTreeIntoMem( char *file_loc );
	void textModeWriteTreeFile( char *file_loc );

private:

	typedef struct NodeSt
	{
		unsigned char *descVector; /*Pointer to the node's descriptor vector*/
	} *Node;

	typedef struct TreeLevelSt
	{
		Node *nodes;	/*Array holding the level's nodes*/
	} *TreeLevel;

	typedef struct VocabTreeSt
	{
		TreeLevel *level;     /* Array of levels */
	} *VocabTree;

	unsigned char dim;    
	unsigned char levels;
	unsigned char clusters;
	bool init;
	VocabTree tree;
};



#endif